document.addEventListener('DOMContentLoaded', function () {
    const form = document.getElementById('zospay-payment-form');
    if (form) {
        form.addEventListener('submit', function (e) {
            // Clear existing error messages
            const existingErrors = document.querySelectorAll('.zospay-error');
            existingErrors.forEach(function (error) {
                error.remove();
            });

            let isValid = true;

            // Card Number Validation
            const cardNumber = document.getElementById('zospay_card_number').value.replace(/\D/g, '');
            if (!luhnCheck(cardNumber)) {
                showError('zospay_card_number', 'Invalid card number.');
                isValid = false;
            }

            // Expiry Month Validation
            const cardMonth = parseInt(document.getElementById('zospay_card_month').value, 10);
            const cardYear = parseInt(document.getElementById('zospay_card_year').value, 10);
            const currentDate = new Date();
            const currentYear = currentDate.getFullYear() % 100; // Last two digits
            const currentMonth = currentDate.getMonth() + 1;

            if (isNaN(cardMonth) || cardMonth < 1 || cardMonth > 12) {
                showError('zospay_card_month', 'Invalid expiration month.');
                isValid = false;
            }

            if (isNaN(cardYear) || cardYear < currentYear || (cardYear === currentYear && cardMonth < currentMonth)) {
                showError('zospay_card_year', 'Your card has expired.');
                isValid = false;
            }

            // CVC Validation
            const cardCvc = document.getElementById('zospay_card_cvc').value.trim();
            if (!/^\d{3,4}$/.test(cardCvc)) {
                showError('zospay_card_cvc', 'Invalid CVC (CVV) number.');
                isValid = false;
            }

            if (!isValid) {
                e.preventDefault();
            }
        });
    }

    function showError(fieldId, message) {
        const field = document.getElementById(fieldId);
        if (field) {
            const error = document.createElement('div');
            error.className = 'zospay-error woocommerce-error';
            error.innerText = message;
            field.parentNode.appendChild(error);
        }
    }

    function luhnCheck(number) {
        let sum = 0;
        let shouldDouble = false;
        for (let i = number.length - 1; i >= 0; i--) {
            let digit = parseInt(number.charAt(i), 10);
            if (shouldDouble) {
                digit *= 2;
                if (digit > 9) digit -= 9;
            }
            sum += digit;
            shouldDouble = !shouldDouble;
        }
        return (sum % 10) === 0;
    }
});
