<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

include_once plugin_dir_path( __FILE__ ) . 'zospay-api.php';

class WC_Gateway_Zospay extends WC_Payment_Gateway {

    public function __construct() {
        $this->id                 = 'zospay';
        $this->method_title       = __( 'Zospay', 'zospay-woocommerce' );
        $this->method_description = __( 'Accept payments through ZosPay Payment Gateway.', 'zospay-woocommerce' );
        $this->has_fields         = false; // Checkout sayfasında ödeme formunu göstermiyoruz
		$this->button_title = $this->get_option( 'button_title', __( 'Pay With Credit Card.', 'zospay-woocommerce' ) );
		
		$iconUrl = plugin_dir_url(dirname(__FILE__));
        $this->icon = $iconUrl . 'assets/img/footer_logos.png';

        // Return URL
        $this->return_url = WC()->api_request_url( 'wc_gateway_zospay_return' );

        // Ayarları yükle
        $this->init_form_fields();
        $this->init_settings();
		$this->supports = array('products', 'refunds');

        $this->title          = $this->get_option( 'title' );
        $this->description    = $this->get_option( 'description' );
        $this->api_key        = $this->get_option( 'api_key' );
        $this->order_status   = $this->get_option( 'order_status', 'default' );


        // Yönetici ayarlarını kaydet
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

        // Callback işlemlerini yönet
        add_action( 'woocommerce_api_wc_gateway_zospay', array( $this, 'handle_callback' ) );

        // Return işlemlerini yönet
        add_action( 'woocommerce_api_wc_gateway_zospay_return', array( $this, 'handle_return' ) );

        // Order Pay sayfasında ödeme formunu göster
        add_action( 'woocommerce_receipt_' . $this->id, array( $this, 'receipt_page' ) );

        // Ödeme formu gönderimini işle
        add_action( 'woocommerce_api_wc_gateway_zospay_process', array( $this, 'process_payment_submission' ) );

        // Hata mesajlarını göstermek için
        add_action( 'woocommerce_before_checkout_form', array( $this, 'show_error_message' ), 10 );
        add_action( 'woocommerce_before_pay_form', array( $this, 'show_error_message' ), 10 );
		add_action( 'woocommerce_before_thankyou', array( $this, 'zospay_display_error_message' ), 20 );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_assets' ) );
    }

    // Yönetici ayar alanları
    public function init_form_fields() {
        $this->form_fields = array(
			 'callback_url' => array(
                'title'       => __( 'Callback URL', 'zospay-woocommerce' ),
                'type'        => 'title',
                'description' => '<p>' . __( '<b>You must add the callback url.</b> Set the following Callback URL in your Zospay account to confirm payments: <a href="https://zospay.com/user/profile/callback" target="_blank">https://zospay.com/user/profile/callback</a>', 'zospay-woocommerce' ) . ': <p><b>YOUR CALLBACK URL:</b></p></p><code>' . esc_url( WC()->api_request_url( 'wc_gateway_zospay' ) ) . '</code>',
            ),
            'enabled'     => array(
                'title'   => __( 'Enable', 'zospay-woocommerce' ),
                'type'    => 'checkbox',
                'label'   => __( 'Enable Zospay payment gateway.', 'zospay-woocommerce' ),
                'default' => 'yes',
            ),
            'title'       => array(
                'title'       => __( 'Title', 'zospay-woocommerce' ),
                'type'        => 'text',
                'description' => __( 'The title displayed during checkout.', 'zospay-woocommerce' ),
                'default'     => __( 'Credit/Debit Cards (Zospay)', 'zospay-woocommerce' ),
            ),
            'description' => array(
                'title'       => __( 'Description', 'zospay-woocommerce' ),
                'type'        => 'textarea',
                'description' => __( 'The description displayed during checkout.', 'zospay-woocommerce' ),
                'default'     => __( 'Pay securely using your credit card.', 'zospay-woocommerce' ),
            ),
			   // Yeni eklenen button_title alanı
        	'button_title' => array(
            	'title'       => __( 'Checkout Button Title', 'zospay-woocommerce' ),
            	'type'        => 'text',
           	 	'description' => __( 'The title of the checkout button.', 'zospay-woocommerce' ),
            	'default'     => __( 'Pay With Credit Card.', 'zospay-woocommerce' ),
            	'desc_tip'    => true,
        	),
            'api_key'     => array(
                'title'       => __( 'API Key', 'zospay-woocommerce' ),
                'type'        => 'text',
                'description' => __( 'Enter your Zospay API key.', 'zospay-woocommerce' ),
                'default'     => '',
            ),
            'order_status' => array(
    			'title'       => __( 'Order Status', 'zospay-woocommerce' ),
    			'type'        => 'select',
    			'description' => __( 'Order status when payment is successful. If set to "Default", the order status will be set based on  									 product types.', 'zospay-woocommerce' ),
    			'default'     => 'default',
    			'options'     => array_merge(
        					  array( 'default' => __( 'Default', 'zospay-woocommerce' ) ),
        		    		  wc_get_order_statuses()
    		),
    		'desc_tip'    => true,
			),
        );
    }

    // Yönetici seçenekleri
    public function admin_options() {
        ?>
        <h2><?php _e( 'Zospay Payment Gateway', 'zospay-woocommerce' ); ?></h2>
        <table class="form-table">
            <?php $this->generate_settings_html(); ?>
        </table>
        <?php
    }
	
/**
 * Özel CSS dosyasını enqueue eder.
 */
public function enqueue_assets() {
        if ( is_checkout() || is_account_page() || is_wc_endpoint_url( 'order-pay' ) ) { // CSS ve JS'i sadece gerekli sayfalarda yükleyin
            // CSS
            wp_enqueue_style(
                'zospay-custom-style', // Stil adı
                plugins_url( 'assets/css/zospay-custom.css', dirname( dirname( __FILE__ ) ) . '/zospay-woocommerce.php' ), // CSS dosyasının yolu
                array(), // Bağımlılıklar
                '1.0.0' // Versiyon
            );

            // JavaScript
            wp_enqueue_script(
                'zospay-validation', // Script adı
                plugins_url( 'assets/js/zospay-validation.js', dirname( dirname( __FILE__ ) ) . '/zospay-woocommerce.php' ), // JS dosyasının yolu
                array( 'jquery' ), // Bağımlılıklar
                '1.0.0', // Versiyon
                true // Footer'da yükle
            );
        }
    }

    // Ödeme sayfasındaki alanlar (Checkout sayfasında görünmeyecek)
    public function payment_fields() {
        // Burayı boş bırakıyoruz veya isterseniz bilgi mesajı koyabilirsiniz
        if ( $this->description ) {
            echo wpautop( wp_kses_post( $this->description ) );
        }
    }

   

    // Ödeme işlemini başlat
    public function process_payment( $order_id ) {
    $order = wc_get_order( $order_id );


    // Kullanıcıyı Order Pay sayfasına yönlendir
    return array(
        'result'   => 'success',
        'redirect' => $order->get_checkout_payment_url( true ),
    );
}

    // Order Pay sayfasında ödeme formunu göster
    public function receipt_page( $order_id ) {
    echo '<p>' . __( 'To complete your payment, please enter the card details below.', 'zospay-woocommerce' ) . '</p>';

    echo '<form method="post" id="zospay-payment-form" action="' . esc_url( WC()->api_request_url( 'wc_gateway_zospay_process' ) ) . '">';
    $this->zospay_payment_fields(); // Ödeme formunu burada gösteriyoruz
    echo '<input type="hidden" name="order_id" value="' . esc_attr( $order_id ) . '" />';
    wp_nonce_field( 'zospay_process_payment', 'zospay_nonce' );
    // Buton başlığını dinamik hale getir
    echo '<button type="submit" class="button alt">' . esc_html( $this->button_title ) . '</button>';
    echo '</form>';
}


    // Ödeme formunu Order Pay sayfasında göstermek için
    // Ödeme formunu Order Pay sayfasında göstermek için
public function zospay_payment_fields() {
    ?>
    <fieldset class="zospay-payment-fields">
        <div class="zospay-form-row full-width">
            <label for="zospay_card_number"><?php _e( 'Card Number', 'zospay-woocommerce' ); ?> <span class="required">*</span></label>
            <input id="zospay_card_number" name="zospay_card_number" type="text" autocomplete="cc-number" inputmode="numeric" placeholder="1234 5678 9012 3456" required>
        </div>
        <div class="zospay-form-row">
            <label for="zospay_card_month"><?php _e( 'Expiration Month', 'zospay-woocommerce' ); ?> <span class="required">*</span></label>
            <input id="zospay_card_month" name="zospay_card_month" type="text" maxlength="2" autocomplete="cc-exp-month" inputmode="numeric" placeholder="MM" required>
        </div>
        <div class="zospay-form-row">
            <label for="zospay_card_year"><?php _e( 'Expiration Year', 'zospay-woocommerce' ); ?> <span class="required">*</span></label>
            <input id="zospay_card_year" name="zospay_card_year" type="text" maxlength="2" autocomplete="cc-exp-year" inputmode="numeric" placeholder="YY" required>
        </div>
        <div class="zospay-form-row">
            <label for="zospay_card_cvc"><?php _e( 'CVC (CVV)', 'zospay-woocommerce' ); ?> <span class="required">*</span></label>
            <input id="zospay_card_cvc" name="zospay_card_cvc" type="text" maxlength="4" autocomplete="cc-csc" inputmode="numeric" placeholder="123" required>
        </div>
    </fieldset>
    <?php
}


    // Ödeme formu gönderimini işle
    public function process_payment_submission() {
        if ( ! isset( $_POST['zospay_nonce'] ) || ! wp_verify_nonce( $_POST['zospay_nonce'], 'zospay_process_payment' ) ) {
            wc_add_notice( __( 'Invalid payment request.', 'zospay-woocommerce' ), 'error' );
            wp_redirect( wc_get_page_permalink( 'cart' ) );
            exit;
        }

        $order_id = isset( $_POST['order_id'] ) ? intval( $_POST['order_id'] ) : 0;
        if ( ! $order_id ) {
            wc_add_notice( __( 'Invalid order ID', 'zospay-woocommerce' ), 'error' );
            wp_redirect( wc_get_page_permalink( 'cart' ) );
            exit;
        }
				
        $order = wc_get_order( $order_id );

        if ( ! $order ) {
            wc_add_notice( __( 'Order not found.', 'zospay-woocommerce' ), 'error' );
            wp_redirect( wc_get_page_permalink( 'cart' ) );
            exit;
        }

        // Ödeme alanlarını doğrula
        if ( ! $this->validate_fields() ) {
            // Doğrulama başarısızsa, kullanıcıyı ödeme sayfasına geri yönlendir
            wp_redirect( $order->get_checkout_payment_url( true ) );
            exit;
        }
		
		     // Adres bileşenlerini al
    $address    = $order->get_billing_address_1();
    $address_2  = $order->get_billing_address_2();
    $city       = $order->get_billing_city();
    $postcode   = $order->get_billing_postcode();

    // Ülke ve eyalet kodlarını alın
    $country_code = $order->get_billing_country();
    $state_code   = $order->get_billing_state();

    // Ülke kodunu tam isme dönüştür
    if ( ! empty( $country_code ) ) {
        $countries = WC()->countries->get_countries();
        $country = isset( $countries[ $country_code ] ) ? $countries[ $country_code ] : $country_code;
    } else {
        $country = '';
    }

    // Eyalet kodunu tam isme dönüştür
    if ( ! empty( $state_code ) ) {
        $states = WC()->countries->get_states( $country_code );
        if ( ! empty( $states ) && isset( $states[ $state_code ] ) ) {
            $state = $states[ $state_code ];
        } else {
            $state = $state_code;
        }
    } else {
        $state = '';
    }

    // Adres bileşenlerini birleştir
    $full_address = $address;
    if ( ! empty( $address_2 ) ) {
        $full_address .= ', ' . $address_2;
    }
    if ( ! empty( $city ) ) {
        $full_address .= ', ' . $city;
    }
    if ( ! empty( $state ) ) {
        $full_address .= ', ' . $state; // Güncellenmiş eyalet ismi
    }
    if ( ! empty( $postcode ) ) {
        $full_address .= ', ' . $postcode;
    }
    if ( ! empty( $country ) ) {
        $full_address .= ', ' . $country; // Güncellenmiş ülke ismi
    }


        // Kart bilgilerini al
        $card_number = sanitize_text_field( $_POST['zospay_card_number'] );
        $card_month  = sanitize_text_field( $_POST['zospay_card_month'] );
        $card_year   = sanitize_text_field( $_POST['zospay_card_year'] );
        $card_cvc    = sanitize_text_field( $_POST['zospay_card_cvc'] );

        // Ödeme verilerini hazırla
        $payment_data = array(
            'amount'       => $order->get_total(),
            'currency'     => get_woocommerce_currency(),
            'order_id'     => $order->get_id(),
            'first_name'   => $order->get_billing_first_name(),
            'last_name'    => $order->get_billing_last_name(),
            'email'        => $order->get_billing_email(),
            'phone'        => $order->get_billing_phone(),
            'address'      => $full_address,
            'card_number'  => $card_number,
            'card_month'   => $card_month,
            'card_year'    => $card_year,
            'card_cvc'     => $card_cvc,
            'return_url'   => $this->return_url,
        );

        // Zospay API'sine istek gönder
        $zospay_api = new Zospay_API( $this->api_key );
        $response   = $zospay_api->create_payment( $payment_data );

        // API yanıtını kontrol et
        if ( isset( $response['redirect_url'] ) ) {
            // 3D Secure yönlendirmesi gerekiyorsa
            wp_redirect( $response['redirect_url'] );
            exit;
        } else {
            if ( isset( $response['status'] ) && ( strtolower( $response['status'] ) === 'successful' || strtolower( $response['status'] ) === 'success' ) ) {
                // Ödeme başarılı
                $order->payment_complete();
                $order->add_order_note( __( 'Payment successful with Zospay.', 'zospay-woocommerce' ) );
				
				// Sepeti boşalt
    			WC()->cart->empty_cart();

                wp_redirect( $this->get_return_url( $order ) );
                exit;
            } else {
                // Ödeme başarısız
                $error_message = isset( $response['message'] ) ? $response['message'] : __( 'An unknown error occurred.', 'zospay-woocommerce' );
                $order->update_status( 'failed', __( 'Ödeme başarısız: ', 'zospay-woocommerce' ) . $error_message );
                wc_add_notice( __( 'Payment error: ', 'zospay-woocommerce' ) . $error_message, 'error' );

                // Hata mesajını göstermek için
                $order->add_meta_data( '_zospay_error_message', $error_message );
                $order->save();

                wp_redirect( $order->get_checkout_payment_url( true ) );
                exit;
            }
        }
    }

    // Hata mesajlarını göstermek için
    public function show_error_message() {
        if ( $notices = wc_get_notices( 'error' ) ) {
            wc_print_notices();
        }
    }

    // Zospay'den gelen callback'i işle
    public function handle_callback() {
    // Zospay'den gelen verileri al
    $order_id = isset( $_POST['order_id'] ) ? sanitize_text_field( $_POST['order_id'] ) : '';
    $status   = isset( $_POST['status'] ) ? sanitize_text_field( $_POST['status'] ) : '';
    $message  = isset( $_POST['message'] ) ? sanitize_text_field( $_POST['message'] ) : '';
    $uuid     = isset( $_POST['payment_id'] ) ? sanitize_text_field( $_POST['payment_id'] ) : '';
    $hash     = isset( $_POST['hash'] ) ? sanitize_text_field( $_POST['hash'] ) : '';
    $type     = isset( $_POST['type'] ) ? sanitize_text_field( $_POST['type'] ) : '';

    $order = wc_get_order( $order_id );

    if ( ! $order ) {
        echo 'Order not found';
        exit;
    }

    // Hash doğrulaması (Zospay dokümantasyonuna göre güncelleyin)
    $calculatedHash = hash_hmac( 'sha256', $this->return_url . $order->get_currency() . $order_id, $this->api_key );
    if ( $hash !== $calculatedHash ) {
        echo 'Invalid hash';
        exit;
    }

    if ( strtolower( $status ) === 'successful' || strtolower( $status ) === 'success' ) {
        // Ödeme başarılı

        if ( $this->order_status == 'default' ) {
            // Sipariş durumunu ürünlerin türüne göre belirle
            $virtual_order = true;
            foreach ( $order->get_items() as $item_id => $item ) {
                $product = $item->get_product();
                if ( ! $product->is_virtual() || ! $product->is_downloadable() ) {
                    $virtual_order = false;
                    break;
                }
            }

            if ( $virtual_order ) {
                // Tüm ürünler dijital ise sipariş durumunu 'completed' yap
                $order_status = 'completed';
            } else {
                // Fiziksel ürünler varsa sipariş durumunu 'processing' yap
                $order_status = 'processing';
            }
        } else {
            // Yönetici tarafından seçilen sipariş durumunu kullan
            $order_status = str_replace( 'wc-', '', $this->order_status );
        }

        // Sipariş durumunu güncelle
        $order->update_status( $order_status, __( 'Payment approved by Zospay. ', 'zospay-woocommerce' ) . $uuid );
        $order->add_order_note( __( 'Payment confirmed by Zospay.  ', 'zospay-woocommerce' ) . $uuid );

        // Stok azalt
        wc_reduce_stock_levels( $order_id );

        // Sepeti boşalt
        WC()->cart->empty_cart();
    } else {
        // Ödeme başarısız
        $order->update_status( 'failed', __( 'Payment failed through Zospay: ', 'zospay-woocommerce' ) . $message );
    }

    // Zospay'e başarılı yanıt gönder
    echo 'OK';
    exit;
}

	
	public function zospay_display_error_message( $order_id ) {
    $order = wc_get_order( $order_id );
    if ( $order && $order->has_status( 'failed' ) ) {
        $message = $order->get_meta( '_zospay_error_message' );
        if ( $message ) {
            echo '<div class="woocommerce-error">';
            echo wpautop( esc_html( $message ) );
            echo '</div>';
        }
    }
}

    // Zospay'den gelen dönüş işlemlerini yönet
    public function handle_return() {
        // Zospay'den gelen verileri al
        $order_id = isset( $_POST['order_id'] ) ? sanitize_text_field( $_POST['order_id'] ) : '';
        $status   = isset( $_POST['status'] ) ? sanitize_text_field( $_POST['status'] ) : '';
        $message  = isset( $_POST['message'] ) ? sanitize_text_field( $_POST['message'] ) : '';
        $uuid     = isset( $_POST['payment_id'] ) ? sanitize_text_field( $_POST['payment_id'] ) : '';

        $order = wc_get_order( $order_id );

        if ( ! $order ) {
            wc_add_notice( __( 'Order not found.', 'zospay-woocommerce' ), 'error' );
            wp_redirect( wc_get_page_permalink( 'cart' ) );
            exit;
        }

        if ( strtolower( $status ) === 'successful' || strtolower( $status ) === 'success' ) {
            // Ödeme başarılı
			WC()->cart->empty_cart();
            wp_redirect( $this->get_return_url( $order ) );
            exit;
        } else {
            // Ödeme başarısız
            // Siparişi başarısız olarak ayarla
            $order->update_status( 'failed', __( 'Payment failed: ', 'zospay-woocommerce' ) . $message );

            // Başarısızlık mesajını sipariş meta verisine kaydet
            $order->update_meta_data( '_zospay_error_message', $message );
            $order->save();

            // Kullanıcıyı order-received sayfasına yönlendir ve başarısızlık parametresini ekle
            $redirect_url = wc_get_endpoint_url( 'order-received', $order->get_id(), wc_get_checkout_url() );
            $redirect_url = add_query_arg( array( 'key' => $order->get_order_key(), 'result' => 'failure' ), $redirect_url );

            wp_redirect( $redirect_url );
            exit;
        }
    }
	
	/**
 * Ödeme alanlarını doğrular.
 *
 * @return bool
 */
/**
 * Ödeme alanlarını doğrular.
 *
 * @return bool
 */
public function validate_fields() {
    // Check if the payment form has been submitted
    if ( $_SERVER['REQUEST_METHOD'] !== 'POST' || ! isset( $_POST['zospay_nonce'] ) ) {
        return true; // Skip validation if form was not submitted
    }

    $is_valid = true;

    // Card Number Validation (Luhn Algorithm)
    if ( isset( $_POST['zospay_card_number'] ) ) {
        $card_number = preg_replace('/\D/', '', $_POST['zospay_card_number']);
        if ( !$this->is_valid_card_number( $card_number ) ) {
            wc_add_notice( __( 'Invalid card number.', 'zospay-woocommerce' ), 'error' );
            $is_valid = false;
        }
    } else {
        wc_add_notice( __( 'Card number is required.', 'zospay-woocommerce' ), 'error' );
        $is_valid = false;
    }

    // Expiry Date Validation
    if ( isset( $_POST['zospay_card_month'], $_POST['zospay_card_year'] ) ) {
        $card_month = intval( $_POST['zospay_card_month'] );
        $card_year  = intval( $_POST['zospay_card_year'] );

        if ( $card_month < 1 || $card_month > 12 ) {
            wc_add_notice( __( 'Invalid expiration month.', 'zospay-woocommerce' ), 'error' );
            $is_valid = false;
        }

        // Add 2000 to the year to make it a full year (e.g., 24 -> 2024)
        $current_year = intval( date( 'y' ) );
        $current_month = intval( date( 'm' ) );
        $full_year = $card_year < 100 ? 2000 + $card_year : $card_year;

        if ( $full_year < intval( date( 'Y' ) ) || ( $full_year === intval( date( 'Y' ) ) && $card_month < $current_month ) ) {
            wc_add_notice( __( 'Your card has expired.', 'zospay-woocommerce' ), 'error' );
            $is_valid = false;
        }
    } else {
        wc_add_notice( __( 'Card expiration date is required.', 'zospay-woocommerce' ), 'error' );
        $is_valid = false;
    }

    // CVC Validation
    if ( isset( $_POST['zospay_card_cvc'] ) ) {
        $card_cvc = preg_replace('/\D/', '', $_POST['zospay_card_cvc']);
        if ( !preg_match('/^\d{3,4}$/', $card_cvc ) ) {
            wc_add_notice( __( 'Invalid CVC (CVV) number.', 'zospay-woocommerce' ), 'error' );
            $is_valid = false;
        }
    } else {
        wc_add_notice( __( 'CVC (CVV) number is required.', 'zospay-woocommerce' ), 'error' );
        $is_valid = false;
    }

    return $is_valid;
}



/**
 * Luhn Algoritmasını kullanarak kart numarasını doğrular.
 *
 * @param string $number Kart numarası
 * @return bool
 */
private function is_valid_card_number( $number ) {
    $number = preg_replace('/\D/', '', $number);
    $checksum = 0;
    $length = strlen( $number );
    $parity = $length % 2;

    for ( $i = 0; $i < $length; $i++ ) {
        $digit = intval( $number[$i] );
        if ( $i % 2 == $parity ) {
            $digit *= 2;
            if ( $digit > 9 ) {
                $digit -= 9;
            }
        }
        $checksum += $digit;
    }

    return ( $checksum % 10 ) === 0;
}

}
